@@ -1,43 +1,52 @@
 import re
 from bisect import bisect
 from io import open
+from pathlib import Path
+from typing import List, Iterable, Pattern, Tuple, Match, Optional
 
 from .example import Example
+from .python import import_path
+from .region import Region
+from .typing import Parser, Evaluator
 
 
-class Document(object):
+class Document:
     """
     This is Sybil's representation of a documentation source file.
     It will be instantiated by Sybil and provided to each parser in turn.
+
+    Different types of document can be handled by subclassing to provide the
+    required :any:`evaluation <evaluator>`. The required file extensions, such as ``'.py'``,
+    can then be mapped to these subclasses using :class:`Sybil's <Sybil>`
+    ``document_types`` parameter.
     """
 
-    #: This can be set by :ref:`evaluators <developing-parsers>` to affect the evaluation
+    #: This can be set by :any:`evaluators <Evaluator>` or
+    #: :class:`subclasses <sybil.document.PythonDocument>` to affect the evaluation
     #: of future examples. It can be set to a callable that takes an
     #: :class:`~sybil.example.Example`. This callable can then do whatever it needs to do,
-    #: including not executing the example at all, modifying it, or the
-    #: :class:`~sybil.document.Document` or calling the original evaluator on the example.
+    #: including not executing the example at all, modifying the :class:`~sybil.example.Example`
+    #: or the :class:`~sybil.document.Document`, or calling the original evaluator on the example.
     #: This last case should always take the form of ``example.region.evaluator(example)``.
-    evaluator = None
+    evaluator: Evaluator = None
 
-    def __init__(self, text, path):
+    def __init__(self, text: str, path: str):
         #: This is the text of the documentation source file.
-        self.text = text
+        self.text: str = text
         #: This is the absolute path of the documentation source file.
-        self.path = path
-        self.end = len(text)
-        self.regions = []
-        #: This dictionary is the namespace in which all example parsed from
+        self.path: str = path
+        self.end: int = len(text)
+        self.regions: List[Tuple[int, Region]] = []
+        #: This dictionary is the namespace in which all examples parsed from
         #: this document will be evaluated.
-        self.namespace = {}
+        self.namespace: dict = {}
 
     @classmethod
-    def parse(cls, path, *parsers, **kw):
+    def parse(cls, path: str, *parsers: Parser, encoding: str = 'utf-8') -> 'Document':
         """
         Read the text from the supplied path and parse it into a document
         using the supplied parsers.
         """
-        # python 2 compat:
-        encoding = kw.pop('encoding', 'utf-8')
         with open(path, encoding=encoding) as source:
             text = source.read()
         document = cls(text, path)
@@ -46,29 +55,29 @@ class Document(object):
                 document.add(region)
         return document
 
-    def line_column(self, position):
+    def line_column(self, position: int) -> str:
         """
-        Return a line and column location in this document based on a byte
+        Return a line and column location in this document based on a character
         position.
         """
         line = self.text.count('\n', 0, position)+1
         col = position - self.text.rfind('\n', 0, position)
         return 'line {}, column {}'.format(line, col)
 
-    def region_details(self, region):
+    def region_details(self, region: Region) -> str:
         return '{!r} from {} to {}'.format(
             region,
             self.line_column(region.start),
             self.line_column(region.end)
         )
 
-    def raise_overlap(self, *regions):
+    def raise_overlap(self, *regions: Region) -> None:
         reprs = []
         for region in regions:
             reprs.append(self.region_details(region))
         raise ValueError('{} overlaps {}'.format(*reprs))
 
-    def add(self, region):
+    def add(self, region: Region) -> None:
         if region.start < 0:
             raise ValueError('{} is before start of document'.format(
                 self.region_details(region)
@@ -89,7 +98,7 @@ class Document(object):
                 self.raise_overlap(region, next)
         self.regions.insert(index, entry)
 
-    def __iter__(self):
+    def __iter__(self) -> Iterable[Example]:
         line = 1
         place = 0
         for _, region in self.regions:
@@ -100,9 +109,11 @@ class Document(object):
                           line, region.start-line_start,
                           region, self.namespace)
 
-    def find_region_sources(self, start_pattern, end_pattern):
+    def find_region_sources(
+        self, start_pattern: Pattern[str], end_pattern: Pattern[str]
+    ) -> Tuple[Match, Match, str]:
         """
-        This helper method can be called used to extract source text
+        This helper method can be used to extract source text
         for regions based on the two :ref:`regular expressions <re-objects>`
         provided.
         
@@ -119,3 +130,22 @@ class Document(object):
             source_end = end_match.start()
             source = self.text[source_start:source_end]
             yield start_match, end_match, source
+
+
+class PythonDocument(Document):
+    """
+    A :class:`~sybil.Document` type that imports the document's source
+    file as a Python module, making names within it available in the document's
+    :attr:`~sybil.Document.namespace`.
+    """
+
+    def evaluator(self, example: Example) -> Optional[str]:
+        """
+        Imports the document's source file as a Python module when the first
+        :class:`~sybil.example.Example` from it is evaluated.
+        """
+        module = import_path(Path(self.path))
+        self.namespace.update(module.__dict__)
+        result = example.region.evaluator(example)
+        self.evaluator = None
+        return result
